@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2016 Red Hat
- * Copyright (C) 2017 Alberts Muktupāvels
+ * Copyright (C) 2017-2019 Alberts Muktupāvels
+ * Copyright (c) 2018 DisplayLink (UK) Ltd.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +41,7 @@ typedef struct
   GfMonitorConfig        *monitor_config;
   GPtrArray              *crtc_infos;
   GPtrArray              *output_infos;
+  GArray                 *reserved_crtcs;
 } MonitorAssignmentData;
 
 typedef enum
@@ -179,8 +181,8 @@ create_preferred_logical_monitor_config (GfMonitorManager           *monitor_man
   switch (layout_mode)
     {
       case GF_LOGICAL_MONITOR_LAYOUT_MODE_LOGICAL:
-        width /= scale;
-        height /= scale;
+        width = (int) roundf (width / scale);
+        height = (int) roundf (height / scale);
         break;
 
       case GF_LOGICAL_MONITOR_LAYOUT_MODE_PHYSICAL:
@@ -405,6 +407,20 @@ create_for_builtin_display_rotation (GfMonitorConfigManager *config_manager,
 
   if (rotate)
     transform = (current_logical_monitor_config->transform + 1) % GF_MONITOR_TRANSFORM_FLIPPED;
+  else
+    {
+      GfMonitor *panel;
+
+      /*
+       * The transform coming from the accelerometer should be applied to
+       * the crtc as is, without taking panel-orientation into account, this
+       * is done so that non panel-orientation aware desktop environments do the
+       * right thing. Mutter corrects for panel-orientation when applying the
+       * transform from a logical-monitor-config, so we must convert here.
+       */
+      panel = gf_monitor_manager_get_laptop_panel (config_manager->monitor_manager);
+      transform = gf_monitor_crtc_to_logical_transform (panel, transform);
+    }
 
   if (current_logical_monitor_config->transform == transform)
     return NULL;
@@ -441,6 +457,22 @@ create_for_builtin_display_rotation (GfMonitorConfigManager *config_manager,
                                  layout_mode, GF_MONITORS_CONFIG_FLAG_NONE);
 }
 
+static gboolean
+is_crtc_reserved (GfCrtc *crtc,
+                  GArray *reserved_crtcs)
+{
+  unsigned int i;
+
+  for (i = 0; i < reserved_crtcs->len; i++)
+    {
+       glong id = g_array_index (reserved_crtcs, glong, i);
+       if (id == crtc->crtc_id)
+         return TRUE;
+    }
+
+  return FALSE;
+}
+
 static gboolean
 is_crtc_assigned (GfCrtc    *crtc,
                   GPtrArray *crtc_infos)
@@ -460,13 +492,34 @@ is_crtc_assigned (GfCrtc    *crtc,
 
 static GfCrtc *
 find_unassigned_crtc (GfOutput  *output,
-                      GPtrArray *crtc_infos)
+                      GPtrArray *crtc_infos,
+                      GArray    *reserved_crtcs)
 {
+  GfCrtc *crtc;
   unsigned int i;
 
+  crtc = gf_output_get_assigned_crtc (output);
+  if (crtc && !is_crtc_assigned (crtc, crtc_infos))
+    return crtc;
+
+  /* then try to assign a CRTC that wasn't used */
   for (i = 0; i < output->n_possible_crtcs; i++)
     {
-      GfCrtc *crtc = output->possible_crtcs[i];
+      crtc = output->possible_crtcs[i];
+
+      if (is_crtc_assigned (crtc, crtc_infos))
+        continue;
+
+      if (is_crtc_reserved (crtc, reserved_crtcs))
+        continue;
+
+      return crtc;
+    }
+
+  /* finally just give a CRTC that we haven't assigned */
+  for (i = 0; i < output->n_possible_crtcs; i++)
+    {
+      crtc = output->possible_crtcs[i];
 
       if (is_crtc_assigned (crtc, crtc_infos))
         continue;
@@ -497,8 +550,8 @@ assign_monitor_crtc (GfMonitor          *monitor,
   gboolean assign_output_as_presentation;
 
   output = monitor_crtc_mode->output;
+  crtc = find_unassigned_crtc (output, data->crtc_infos, data->reserved_crtcs);
 
-  crtc = find_unassigned_crtc (output, data->crtc_infos);
   if (!crtc)
     {
       GfMonitorSpec *monitor_spec = gf_monitor_get_spec (monitor);
@@ -511,10 +564,9 @@ assign_monitor_crtc (GfMonitor          *monitor,
     }
 
   transform = data->logical_monitor_config->transform;
-  if (gf_monitor_manager_is_transform_handled (data->monitor_manager,
-                                               crtc, transform))
-    crtc_transform = transform;
-  else
+  crtc_transform = gf_monitor_logical_to_crtc_transform (monitor, transform);
+  if (!gf_monitor_manager_is_transform_handled (data->monitor_manager,
+                                                crtc, crtc_transform))
     crtc_transform = GF_MONITOR_TRANSFORM_NORMAL;
 
   gf_monitor_calculate_crtc_pos (monitor, mode, output, crtc_transform,
@@ -581,6 +633,7 @@ assign_monitor_crtcs (GfMonitorManager        *manager,
                       GfMonitorConfig         *monitor_config,
                       GPtrArray               *crtc_infos,
                       GPtrArray               *output_infos,
+                      GArray                  *reserved_crtcs,
                       GError                 **error)
 {
   GfMonitorSpec *monitor_spec = monitor_config->monitor_spec;
@@ -616,7 +669,8 @@ assign_monitor_crtcs (GfMonitorManager        *manager,
     .logical_monitor_config = logical_monitor_config,
     .monitor_config = monitor_config,
     .crtc_infos = crtc_infos,
-    .output_infos = output_infos
+    .output_infos = output_infos,
+    .reserved_crtcs = reserved_crtcs
   };
 
   if (!gf_monitor_mode_foreach_crtc (monitor, monitor_mode,
@@ -632,6 +686,7 @@ assign_logical_monitor_crtcs (GfMonitorManager        *manager,
                               GfLogicalMonitorConfig  *logical_monitor_config,
                               GPtrArray               *crtc_infos,
                               GPtrArray               *output_infos,
+                              GArray                  *reserved_crtcs,
                               GError                 **error)
 {
   GList *l;
@@ -644,7 +699,7 @@ assign_logical_monitor_crtcs (GfMonitorManager        *manager,
                                  logical_monitor_config,
                                  monitor_config,
                                  crtc_infos, output_infos,
-                                 error))
+                                 reserved_crtcs, error))
         return FALSE;
     }
 
@@ -752,10 +807,38 @@ gf_monitor_config_manager_assign (GfMonitorManager  *manager,
 {
   GPtrArray *crtc_infos;
   GPtrArray *output_infos;
+  GArray *reserved_crtcs;
   GList *l;
 
   crtc_infos = g_ptr_array_new_with_free_func ((GDestroyNotify) gf_crtc_info_free);
   output_infos = g_ptr_array_new_with_free_func ((GDestroyNotify) gf_output_info_free);
+  reserved_crtcs = g_array_new (FALSE, FALSE, sizeof (glong));
+
+  for (l = config->logical_monitor_configs; l; l = l->next)
+    {
+      GfLogicalMonitorConfig *logical_monitor_config = l->data;
+      GList *k;
+
+      for (k = logical_monitor_config->monitor_configs; k; k = k->next)
+        {
+          GfMonitorConfig *monitor_config = k->data;
+          GfMonitorSpec *monitor_spec = monitor_config->monitor_spec;
+          GfMonitor *monitor;
+          GList *o;
+
+          monitor = gf_monitor_manager_get_monitor_from_spec (manager, monitor_spec);
+
+          for (o = gf_monitor_get_outputs (monitor); o; o = o->next)
+            {
+              GfOutput *output = o->data;
+              GfCrtc *crtc;
+
+              crtc = gf_output_get_assigned_crtc (output);
+              if (crtc)
+                g_array_append_val (reserved_crtcs, crtc->crtc_id);
+            }
+        }
+    }
 
   for (l = config->logical_monitor_configs; l; l = l->next)
     {
@@ -763,10 +846,11 @@ gf_monitor_config_manager_assign (GfMonitorManager  *manager,
 
       if (!assign_logical_monitor_crtcs (manager, logical_monitor_config,
                                          crtc_infos, output_infos,
-                                         error))
+                                         reserved_crtcs, error))
         {
           g_ptr_array_free (crtc_infos, TRUE);
           g_ptr_array_free (output_infos, TRUE);
+          g_array_free (reserved_crtcs, TRUE);
           return FALSE;
         }
     }
@@ -774,6 +858,8 @@ gf_monitor_config_manager_assign (GfMonitorManager  *manager,
   *out_crtc_infos = crtc_infos;
   *out_output_infos = output_infos;
 
+  g_array_free (reserved_crtcs, TRUE);
+
   return TRUE;
 }
 
@@ -981,7 +1067,11 @@ GfMonitorsConfig *
 gf_monitor_config_manager_create_for_switch_config (GfMonitorConfigManager    *config_manager,
                                                     GfMonitorSwitchConfigType  config_type)
 {
-  GfMonitorManager *monitor_manager = config_manager->monitor_manager;
+  GfMonitorManager *monitor_manager;
+  GfMonitorsConfig *config;
+
+  monitor_manager = config_manager->monitor_manager;
+  config = NULL;
 
   if (!gf_monitor_manager_can_switch_config (monitor_manager))
     return NULL;
@@ -989,16 +1079,20 @@ gf_monitor_config_manager_create_for_switch_config (GfMonitorConfigManager    *c
   switch (config_type)
     {
       case GF_MONITOR_SWITCH_CONFIG_ALL_MIRROR:
-        return create_for_switch_config_all_mirror (config_manager);
+        config = create_for_switch_config_all_mirror (config_manager);
+        break;
 
       case GF_MONITOR_SWITCH_CONFIG_ALL_LINEAR:
-        return gf_monitor_config_manager_create_linear (config_manager);
+        config = gf_monitor_config_manager_create_linear (config_manager);
+        break;
 
       case GF_MONITOR_SWITCH_CONFIG_EXTERNAL:
-        return create_for_switch_config_external (config_manager);
+        config = create_for_switch_config_external (config_manager);
+        break;
 
       case GF_MONITOR_SWITCH_CONFIG_BUILTIN:
-        return create_for_switch_config_builtin (config_manager);
+        config = create_for_switch_config_builtin (config_manager);
+        break;
 
       case GF_MONITOR_SWITCH_CONFIG_UNKNOWN:
       default:
@@ -1006,7 +1100,10 @@ gf_monitor_config_manager_create_for_switch_config (GfMonitorConfigManager    *c
         break;
     }
 
-  return NULL;
+  if (config)
+    gf_monitors_config_set_switch_config (config, config_type);
+
+  return config;
 }
 
 void
