@@ -36,8 +36,7 @@
 #define SCALE_FACTORS_PER_INTEGER 4
 #define MINIMUM_SCALE_FACTOR 1.0f
 #define MAXIMUM_SCALE_FACTOR 4.0f
-#define MINIMUM_LOGICAL_WIDTH 800
-#define MINIMUM_LOGICAL_HEIGHT 480
+#define MINIMUM_LOGICAL_AREA (800 * 480)
 #define MAXIMUM_REFRESH_RATE_DIFF 0.001f
 
 /* The minimum screen height at which we turn on a window-scale of 2;
@@ -296,6 +295,13 @@ out:
   return scale;
 }
 
+static gboolean
+is_logical_size_large_enough (gint width,
+                              gint height)
+{
+  return width * height >= MINIMUM_LOGICAL_AREA;
+}
+
 static gfloat
 get_closest_scale_factor_for_resolution (gfloat width,
                                          gfloat height,
@@ -316,8 +322,7 @@ get_closest_scale_factor_for_resolution (gfloat width,
 
   if (scale < MINIMUM_SCALE_FACTOR ||
       scale > MAXIMUM_SCALE_FACTOR ||
-      floorf (scaled_w) < MINIMUM_LOGICAL_WIDTH ||
-      floorf (scaled_h) < MINIMUM_LOGICAL_HEIGHT)
+      !is_logical_size_large_enough (floorf (scaled_w), floorf (scaled_h)))
     goto out;
 
   if (floorf (scaled_w) == scaled_w && floorf (scaled_h) == scaled_h)
@@ -1198,6 +1203,22 @@ gf_monitor_mode_foreach_output (GfMonitor          *monitor,
   return TRUE;
 }
 
+gboolean
+gf_monitor_mode_should_be_advertised (GfMonitorMode *monitor_mode)
+{
+  GfMonitorMode *preferred_mode;
+
+  g_return_val_if_fail (monitor_mode != NULL, FALSE);
+
+  preferred_mode = gf_monitor_get_preferred_mode (monitor_mode->monitor);
+  if (monitor_mode->spec.width == preferred_mode->spec.width &&
+      monitor_mode->spec.height == preferred_mode->spec.height)
+    return TRUE;
+
+  return is_logical_size_large_enough (monitor_mode->spec.width,
+                                       monitor_mode->spec.height);
+}
+
 gboolean
 gf_verify_monitor_mode_spec (GfMonitorModeSpec  *mode_spec,
                              GError            **error)
