@@ -3,7 +3,7 @@
  * Copyright (C) 2003 Rob Adams
  * Copyright (C) 2004-2006 Elijah Newren
  * Copyright (C) 2013 Red Hat Inc.
- * Copyright (C) 2017 Alberts Muktupāvels
+ * Copyright (C) 2017-2019 Alberts Muktupāvels
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,9 +27,9 @@
 
 #include <glib-object.h>
 
+#include "gf-gpu-private.h"
 #include "gf-monitor-manager-enums-private.h"
 #include "gf-monitor-manager-types-private.h"
-#include "gf-monitor-manager.h"
 
 G_BEGIN_DECLS
 
@@ -55,70 +55,74 @@ typedef struct
 
 struct _GfOutput
 {
-  GObject            parent;
+  GObject              parent;
 
-  GfMonitorManager  *monitor_manager;
-
-  /* The CRTC driving this output, NULL if the output is not enabled */
-  GfCrtc            *crtc;
+  GfGpu               *gpu;
 
   /* The low-level ID of this output, used to apply back configuration */
-  glong              winsys_id;
-  gchar             *name;
-  gchar             *vendor;
-  gchar             *product;
-  gchar             *serial;
-  gint               width_mm;
-  gint               height_mm;
+  glong                winsys_id;
+  gchar               *name;
+  gchar               *vendor;
+  gchar               *product;
+  gchar               *serial;
+  gint                 width_mm;
+  gint                 height_mm;
 
-  GfConnectorType    connector_type;
+  GfConnectorType      connector_type;
+  GfMonitorTransform   panel_orientation_transform;
 
-  GfCrtcMode        *preferred_mode;
-  GfCrtcMode       **modes;
-  guint              n_modes;
+  GfCrtcMode          *preferred_mode;
+  GfCrtcMode         **modes;
+  guint                n_modes;
 
-  GfCrtc           **possible_crtcs;
-  guint              n_possible_crtcs;
+  GfCrtc             **possible_crtcs;
+  guint                n_possible_crtcs;
 
-  GfOutput         **possible_clones;
-  guint              n_possible_clones;
+  GfOutput           **possible_clones;
+  guint                n_possible_clones;
 
-  gint               backlight;
-  gint               backlight_min;
-  gint               backlight_max;
+  gint                 backlight;
+  gint                 backlight_min;
+  gint                 backlight_max;
 
   /* Used when changing configuration */
-  gboolean           is_dirty;
+  gboolean             is_dirty;
 
   /* The low-level bits used to build the high-level info in GfLogicalMonitor */
-  gboolean           is_primary;
-  gboolean           is_presentation;
-  gboolean           is_underscanning;
-  gboolean           supports_underscanning;
+  gboolean             is_primary;
+  gboolean             is_presentation;
+  gboolean             is_underscanning;
+  gboolean             supports_underscanning;
 
-  gpointer           driver_private;
-  GDestroyNotify     driver_notify;
+  gpointer             driver_private;
+  GDestroyNotify       driver_notify;
 
   /* Get a new preferred mode on hotplug events, to handle
    * dynamic guest resizing
    */
-  gboolean           hotplug_mode_update;
-  gint               suggested_x;
-  gint               suggested_y;
+  gboolean             hotplug_mode_update;
+  gint                 suggested_x;
+  gint                 suggested_y;
 
-  GfTileInfo         tile_info;
+  GfTileInfo           tile_info;
 };
 
 #define GF_TYPE_OUTPUT (gf_output_get_type ())
 G_DECLARE_FINAL_TYPE (GfOutput, gf_output, GF, OUTPUT, GObject)
 
-GfMonitorManager *gf_output_get_monitor_manager (GfOutput *output);
+GfGpu    *gf_output_get_gpu           (GfOutput *output);
 
+void      gf_output_assign_crtc       (GfOutput *output,
+                                       GfCrtc   *crtc);
 
-void              gf_output_parse_edid          (GfOutput *output,
-                                                 GBytes   *edid);
+void      gf_output_unassign_crtc     (GfOutput *output);
 
-gboolean          gf_output_is_laptop           (GfOutput *output);
+GfCrtc   *gf_output_get_assigned_crtc (GfOutput *output);
+
+void      gf_output_parse_edid        (GfOutput *output,
+                                       GBytes   *edid);
+
+gboolean  gf_output_is_laptop         (GfOutput *output);
 
 G_END_DECLS
 
