@@ -6,7 +6,7 @@
  * Copyright (C) 2003 Rob Adams
  * Copyright (C) 2004-2006 Elijah Newren
  * Copyright (C) 2013 Red Hat Inc.
- * Copyright (C) 2017 Alberts Muktupāvels
+ * Copyright (C) 2017-2019 Alberts Muktupāvels
  * Copyright (C) 2017 Red Hat
  *
  * This program is free software: you can redistribute it and/or modify
@@ -30,7 +30,27 @@
 #include "gf-edid-private.h"
 #include "gf-output-private.h"
 
-G_DEFINE_TYPE (GfOutput, gf_output, G_TYPE_OBJECT)
+typedef struct
+{
+  /* The CRTC driving this output, NULL if the output is not enabled */
+  GfCrtc *crtc;
+} GfOutputPrivate;
+
+G_DEFINE_TYPE_WITH_PRIVATE (GfOutput, gf_output, G_TYPE_OBJECT)
+
+static void
+gf_output_dispose (GObject *object)
+{
+  GfOutput *output;
+  GfOutputPrivate *priv;
+
+  output = GF_OUTPUT (object);
+  priv = gf_output_get_instance_private (output);
+
+  g_clear_object (&priv->crtc);
+
+  G_OBJECT_CLASS (gf_output_parent_class)->dispose (object);
+}
 
 static void
 gf_output_finalize (GObject *object)
@@ -60,6 +80,7 @@ gf_output_class_init (GfOutputClass *output_class)
 
   object_class = G_OBJECT_CLASS (output_class);
 
+  object_class->dispose = gf_output_dispose;
   object_class->finalize = gf_output_finalize;
 }
 
@@ -68,10 +89,43 @@ gf_output_init (GfOutput *output)
 {
 }
 
-GfMonitorManager *
-gf_output_get_monitor_manager (GfOutput *output)
+GfGpu *
+gf_output_get_gpu (GfOutput *output)
 {
-  return output->monitor_manager;
+  return output->gpu;
+}
+
+void
+gf_output_assign_crtc (GfOutput *output,
+                       GfCrtc   *crtc)
+{
+  GfOutputPrivate *priv;
+
+  priv = gf_output_get_instance_private (output);
+
+  g_assert (crtc);
+
+  g_set_object (&priv->crtc, crtc);
+}
+
+void
+gf_output_unassign_crtc (GfOutput *output)
+{
+  GfOutputPrivate *priv;
+
+  priv = gf_output_get_instance_private (output);
+
+  g_clear_object (&priv->crtc);
+}
+
+GfCrtc *
+gf_output_get_assigned_crtc (GfOutput *output)
+{
+  GfOutputPrivate *priv;
+
+  priv = gf_output_get_instance_private (output);
+
+  return priv->crtc;
 }
 
 void
